// Tutorial 5 "Texture Mapping"
// gedownloaded von www.codeworx.org
// Version 1.0, Hans-Jakob Schwer, 25.11.2001
// Original von Jeff Molofee, neu kommentiert und bersetzt aus dem Englischen

#include <windows.h>  // diverse Windowsfunktionen
#include <stdio.h>    // Dateien sollen verwendet werden
#include <gl\glu.h>   // Damit kann Glu32 genutzt werden.
#include <gl\gl.h>    // Damit kann OpenGL32 genutzt werden.
#include <gl\glaux.h> // Und das Gleiche nochmal fr Glaux 

HGLRC hRC=NULL; // Der OpenGL Rendering Context
HDC hDC=NULL;   // Gerte Kontext
HWND hWnd=NULL; // Verweist spter auf den Windows Handle
HINSTANCE hInstance; // Die Instanz der Anwendung

bool keys[256]; // Vektor (Array) der den Status 
                // einzelner Tasten enthlt 
                // (gedrckt/nicht gedrckt)

bool active=TRUE; // Wenn active FALSE ist, wurde das 
                  // Fenster vom Benutzer minimiert.

bool fullscreen=TRUE; // Luft das Programm im Vollbildmodus 
                      // oder nicht?

GLfloat xrot;      // Rotation um die X-Achse
GLfloat yrot;      // Rotation um die Y-Achse
GLfloat zrot;      // Rotation um die Z-Achse

GLuint texture[1]; // Texturspeicher

LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);    
// WndProc wird deklariert

GLvoid ReSizeGLScene(GLsizei width, GLsizei height) 
// Initialisierung des OpenGL-Fensters
{
  if (height==0)
  {      
    height=1;    
  }

  // height darf nicht 0 sein, damit es im spteren 
  // Programmablauf nicht zu einer Division durch 0 kommt.

  glViewport(0, 0, width, height);
  // Hier wird der Mittelpunkt auf den die Perspektive zuluft 
  // zurckgesetzt.

  glMatrixMode(GL_PROJECTION); 
  // Hier wird die Projektionsmatrix festgelegt
   
  glLoadIdentity(); 
  // und angepasst

  gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f); 
  // Hier wird die das Verhltnis der Hhe zur Breite bergeben
  // und der Verzerrungswinkel von 45 Grad bergeben

  glMatrixMode(GL_MODELVIEW); 
  // Hier wird die sogenannte modelview-Matrix festgelegt
  
  glLoadIdentity(); 
  // und angepasst.
}

AUX_RGBImageRec *LoadBMP(char *Filename) // Ldt ein BMP
{
  FILE *File=NULL; // Das File Handle

  if (!Filename) 
  // Ist ein Dateiname bergeben worden?

  {
    return NULL; // offenbar nicht... Abbruch
  }

  File=fopen(Filename,"r"); 
  // Versuch die Datei zu ffnen

  if (File) // Existiert die Datei?
  {
    fclose(File); // Datei schlieen
    return auxDIBImageLoad(Filename); 
    // BMP laden und Zeiger (Pointer) zurckgeben, 
    // der auf die Bilddaten verweist
  }
  return NULL; // Laden hat nicht geklappt
}


int LoadGLTextures() // Bitmaps laden und konvertieren
{
  int Status=FALSE; 
  // Status ersteinmal auf FALSE setzen

  AUX_RGBImageRec *TextureImage[1]; 
  // Speicherplatz fr die Textur schaffen

  memset(TextureImage,0,sizeof(void *)*1); 
  //Vorher sichergehen das dieser auch leer ist

  if (TextureImage[0]=LoadBMP("Data/codeworx.bmp"))
  // Bitmap laden
  {
    Status=TRUE; // hat geklappt.

    glGenTextures(1, &texture[0]); // Textur wird erstellt

    glBindTexture(GL_TEXTURE_2D, texture[0]);
    //Es handelt sich um eine 2D Textur

    glTexImage2D(GL_TEXTURE_2D, 0, 3, TextureImage[0]->sizeX, 
                 TextureImage[0]->sizeY, 0, GL_RGB, 
                 GL_UNSIGNED_BYTE, TextureImage[0]->data);

    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    // GL_LINEAR wird fr GL_TEXTURE_MIN_FILTER genutzt
   
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    // GL_LINEAR wird fr GL_TEXTURE_MAG_FILTER genutzt
  }

  if (TextureImage[0]) // Existiert TextureImage[0]?
  {
    if (TextureImage[0]->data) // enthlt sie Bilddaten?
    {
      free(TextureImage[0]->data); // wenn ja, dann diese Bilddaten lschen
    }

    free(TextureImage[0]); // Und auch die Struktur an sich lschen.
  }

  return Status; // Status zurckgeben
}

int InitGL(GLvoid)
{
  if (!LoadGLTextures())                            
  {
    return FALSE; // Texturen konnten nicht geladen werden...
  }

  glEnable(GL_TEXTURE_2D); // Texture Mapping aktivieren

  glShadeModel(GL_SMOOTH); 
  // Das Smooth Shading wird aktiviert, das 
  // sehr schne Farbbergnge auf Krpern ermglicht. 

  glClearColor(0.0f, 0.0f, 0.0f, 0.0f); 
  // In diesem Falle ein schwarzer Hintergrund

  glClearDepth(1.0f); 
  // depht buffer zurckgesetzt

  glEnable(GL_DEPTH_TEST); 
  // Aktivierung des depht Tests (dazu spter mehr.)

  glDepthFunc(GL_LEQUAL); 
  // Der Typ des Depht Tests wird festgelegt

  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST); 
  // Perspektive wird festgelegt

  return TRUE; // Initialisierung scheint geklappt zu haben!
}


int DrawGLScene(GLvoid)
{
   
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  // Die vorherige Szene wird vom Bildschirm gelscht,
  // damit die neuen nicht einfach ber die alten 
  // Objekte gezeichnet werden 
   
  glLoadIdentity();
  // modelview-Matrix wird wieder einmal zurckgesetzt

  glTranslatef(0.0f,0.0f,-5.0f);
  // Damit der Wrfel auch zusehen ist, wird er um 5 Einheiten zurckgesetzt

  glRotatef(xrot,1.0f,0.0f,0.0f); // Um die X Achse rotieren
  glRotatef(yrot,0.0f,1.0f,0.0f); // Um die Y Achse rotieren
  glRotatef(zrot,0.0f,0.0f,1.0f); // Um die Z Achse rotieren


  glBegin(GL_QUADS);

    // Das vordere QUAD
    glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f, 1.0f); 
    // unten links an der Form und der Textur
    glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f, 1.0f); 
    // unten rechts an der Form und der Textur
    glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f, 1.0f, 1.0f); 
    // oben rechts an der Form und der Textur
    glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f, 1.0f, 1.0f); 
    // oben links an der Form und der Textur

    // Das hintere QUAD
    glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
    // unten rechts an der Form und der Textur
    glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f, 1.0f, -1.0f);
    // oben rechts an der Form und der Textur
    glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f, 1.0f, -1.0f);
    // oben links an der Form und der Textur
    glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
    // unten links an der Form und der Textur

    // Das obere QUAD
    glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f, 1.0f, -1.0f); 
    // oben links an der Form und der Textur
    glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, 1.0f, 1.0f); 
    // unten links an der Form und der Textur
    glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, 1.0f, 1.0f); 
    // unten rechts an der Form und der Textur
    glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f, 1.0f, -1.0f); 
    // oben rechts an der Form und der Textur

    // Das untere QUAD
    glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f, -1.0f, -1.0f); 
    // oben rechts an der Form und der Textur
    glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
    // oben links an der Form und der Textur
    glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f, 1.0f); 
    // unten links an der Form und der Textur
    glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f, 1.0f); 
    // unten rechts an der Form und der Textur

    // Das rechte QUAD
    glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f); 
    // unten rechts an der Form und der Textur
    glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f, 1.0f, -1.0f); 
    // oben rechts an der Form und der Textur
    glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f, 1.0f, 1.0f); 
    // oben links an der Form und der Textur
    glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f, 1.0f); 
    // unten links an der Form und der Textur

    // Das linke QUAD
    glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f); 
    // unten links an der Form und der Textur
    glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f, 1.0f); 
    // unten rechts an der Form und der Textur
    glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f, 1.0f, 1.0f); 
    // oben rechts an der Form und der Textur
    glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f, 1.0f, -1.0f); 
    // oben links an der Form und der Textur

  glEnd();

  xrot+=0.4f; // X Achsen Rotation
  yrot+=0.6f; // Y Achsen Rotation
  zrot+=0.1f; // Z Achsen Rotation

  return true; // Alles hat geklappt!
}

GLvoid KillGLWindow(GLvoid)
{

  if (fullscreen) // Kontrolle auf Vollbildmodus
  {
    ChangeDisplaySettings(NULL,0); // Zurck zum Desktop
    ShowCursor(TRUE); // Der abgeschaltete Mauszeiger 
                      // wird wieder angezeigt (Nicht 
                      // vergessen ;)
  }

  if (hRC) // Rendering Context (RC) vorhanden?
  {
    if (!wglMakeCurrent(NULL,NULL)) // Kann der DC und RC berhaupt 
                                    // gelscht werden?
    {
      MessageBox(NULL,"Entfernen des DC und RC fehlgeschlagen.","Fehler",MB_OK | MB_ICONINFORMATION);
    }
   
    if (!wglDeleteContext(hRC)) // Kann der RC gelscht werden?
    {
      MessageBox(NULL,"Entfernen des RC fehlgeschlagen.","Fehler...",MB_OK | MB_ICONINFORMATION);
    }
      hRC=NULL; // Der RC wird NULL gesetzt, also entfernt
    }

   if (hDC && !ReleaseDC(hWnd,hDC)) 
   // Kann der Device Context (DC) freigegeben werden?
   {
     MessageBox(NULL,"Freigabe des Device Context fehlgeschlagen.","Fehler",MB_OK | MB_ICONINFORMATION);

     hDC=NULL; // Der DC wird entfernt
   }

   if (hWnd && !DestroyWindow(hWnd)) 
   // Kann das Programmfenster geschlossen werden?
   {
     MessageBox(NULL,"Konnte hWnd nicht lschen.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);

     hWnd=NULL; // Setze den hWnd auf NULL
   }

   if (!UnregisterClass("OpenGL",hInstance)) 
   // Kann die Registrierung rckgngig gemacht werden?

   {
     MessageBox(NULL,"Konnte Klasse nicht entfernen.","SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);

     hInstance=NULL; // Setze hInstance auf NULL
   }
}

BOOL CreateGLWindow(char* title, int width, int height, int bits, bool fullscreenflag)
{

  GLuint PixelFormat; // Speichert das Pixelformat


  WNDCLASS wc; // wc wird eine Instanz der Fensterklasse


  DWORD dwExStyle; // weitere Informationen
  DWORD dwStyle; // Fensterinformationen


  RECT WindowRect; 
  // Speicher fr aktuelle Auflsung
  
  WindowRect.left=(long)0; 
  // Die linke Seite des Rechtecks wirtd auf 0 gesetzt

  WindowRect.right=(long)width; 
  // Hier wird die gewnschte Breite des Fensters gespeichert

  WindowRect.top=(long)0; 
  // Die obere Seite wird auch auf 0 gesetzt

  WindowRect.bottom=(long)height; 
  // Und hier wird die Hhe abgelegt


  fullscreen=fullscreenflag; 
  // Hier wird fullscreen 
  // auf den Wert von fullscreenflag 
  // gesetzt, welches ja bergeben wurde

  hInstance = GetModuleHandle(NULL); 
  // Die Instanz des Programmes bekommt ein 
  // Handle zugeordnet

  wc.style = CS_HREDRAW | CS_VREDRAW | CS_OWNDC; 
  // Bei Vernderungen in der Hhe und/oder Breite, 
  // soll ne gezeichnet werden

  wc.lpfnWndProc = (WNDPROC) WndProc; 
  // WndProc behandelt die auftretenden Nachrichten

  wc.cbClsExtra = 0; // Wird nicht bentigt
  wc.cbWndExtra = 0; // und das auch nicht
  wc.hInstance = hInstance; // Die Instanz wird festgelegt
  wc.hIcon = LoadIcon(NULL, IDI_WINLOGO); 
  // Ldt das Standardsymbol
  wc.hCursor = LoadCursor(NULL, IDC_ARROW); 
  // Ldt einen Cursor
  wc.hbrBackground = NULL; 
  // Es soll kein bestimmter Hintergrund angezeigt werden
  wc.lpszMenuName = NULL; // Auch ein Men wird nicht bentigt.
  wc.lpszClassName = "OpenGL"; // OpenGL wird der Name der Klasse

  if (!RegisterClass(&wc)) // Versuch die Klasse zu registrieren
  {
    MessageBox(NULL,"Konnte die Fensterklasse nicht registrieren.","ERROR",MB_OK|MB_ICONEXCLAMATION);
    return FALSE; // FALSE zurckgeben und beenden
  }

  if (fullscreen) // Soll im Vollbildmodus gestartet werden
  {

    DEVMODE dmScreenSettings; 
    // Instanz von DEVMODE wird erzeugt

    memset(&dmScreenSettings,0,sizeof(dmScreenSettings)); 
    // Diese wird geleert

    dmScreenSettings.dmSize=sizeof(dmScreenSettings); 
    // dmsize soll genauso gro wie die dmScreenSettings sein

    dmScreenSettings.dmPelsWidth = width; 
    // Die drei Werte (height, width und bits) 
    // wurden der Prozedur bergeben und werden 
    // nun in dmScreenSettings gespeichert

    dmScreenSettings.dmPelsHeight = height;
    dmScreenSettings.dmBitsPerPel = bits; 
    dmScreenSettings.dmFields=DM_BITSPERPEL|
    DM_PELSWIDTH|DM_PELSHEIGHT; 
    // Hier werden die drei Werte in einem Ausdruck gespeichert

    if (ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)
        !=DISP_CHANGE_SUCCESSFUL)    
    {

    // CDS_FULLSCREEN blendet zustzlich die Startleiste aus

      if (MessageBox(NULL,"Der gewnschte Vollbildmodus wird nicht untersttzt, soll stattdessen im Fenstermodus ausgegeben werden?","OpenGL",MB_YESNO|MB_ICONEXCLAMATION)==IDYES)
      {
        fullscreen=FALSE; 
        // Der Benutzer mchte im Fenster weitermachen, 
        // dazu wird fullscreen auf FALSE gesetzt
      }

      else
      {
        return FALSE;
        // Falls der Benutzer das Programm aus gegebenen 
        // Anlass beenden will, wird FALSE zurckgegeben.
      }
    }
  }

  if (fullscreen) 
  // Konnte in den Vollbildmodus geschaltet werden? 
  // (Wenn nicht, wird ja im Fenster weitergemacht!)
  {
    dwExStyle=WS_EX_APPWINDOW; // Fenstereigenschaften
    dwStyle=WS_POPUP; // -"-
    ShowCursor(FALSE); // Der Mauszeiger wird nicht angezeigt
  }

  else
  {

    dwExStyle=WS_EX_APPWINDOW | WS_EX_WINDOWEDGE; 
    // Das Fenster soll zustzlich einen 3D Rahmen bekommen

    dwStyle=WS_OVERLAPPEDWINDOW; 
    // Ein typisches Windowsfenster mit 
    // Minimieren, Maximieren, etc
  }

  AdjustWindowRectEx(&WindowRect, dwStyle, FALSE, dwExStyle); 
  // Fenster wird angepasst

  if (!(hWnd=CreateWindowEx( dwExStyle, 
  // Die erweiterten Eigenschaften des    Fensters
  "OpenGL", // Der Name der Klasse
  title, // Der Titel des Fensters
  WS_CLIPSIBLINGS | // Wird von OpenGL bentigt
  WS_CLIPCHILDREN | // Wird auch von OpenGL bentigt
  dwStyle, // auch Eigenschaften des Fensters
  0, 0, // Die Position des zu erstellenden Fensters
  WindowRect.right-WindowRect.left, 
  // Hier werden die ermittelten Werte fr die Breite eingesetzt
  WindowRect.bottom-WindowRect.top, // und hier fr die Lnge
  NULL, // Es soll kein bergordnetes Fendster erstellt werden
  NULL, // kein Men
  hInstance, // Die Instanz wird bergeben
  NULL))) // Wird nicht bentigt


  {
    KillGLWindow(); // Grafikeinstellungen zurcksetzen
    MessageBox(NULL,"Fenster konnte nicht erstellt werden.","ERROR",MB_OK|MB_ICONEXCLAMATION);
    return FALSE; 
  }



  static PIXELFORMATDESCRIPTOR pfd= 
  // pdf ist jetzt ein PIXELFORMATDESCRIPTOR
  {
    sizeof(PIXELFORMATDESCRIPTOR), 
    // Die Gre muss natrlich stimmen
    1, // Versionsnummer
    PFD_DRAW_TO_WINDOW | 
    // Das Format muss in Fenster sichtbar sein knnen
    PFD_SUPPORT_OPENGL | 
    // OpenGL muss untersttzt werden
    PFD_DOUBLEBUFFER, 
    // Double Buffering muss untersttzt werden
    PFD_TYPE_RGBA, 
    // Das RGBA (Rot,Grn,Blau,Alpha(Transparenz)) 
    // muss untersttzt werden
    bits, 
    // Die Farbtiefe, die schon 
    // bergeben wurde, wird hier bentigt
    0, 0, 0, 0, 0, 0, // wird nicht bentigt
    0, // kein Alpha Buffer
    0, // Shift Bit ignoriert
    0, // kein Accumulation Buffer
    0, 0, 0, 0, // nicht bentigt
    16, // 16Bit Z-Buffer (Depth Buffer)
    0, // kein Stencil Buffer
    0, // kein Auxiliary Buffer
    PFD_MAIN_PLANE, 
    // Die Hauptebene auf die spter gezeichnet wird
    0, // unwichtig
    0, 0, 0 // keine Ebenenmasken bentigt
  };


 if (!(hDC=GetDC(hWnd))) // Versuch, den DC zu bekommen
   {
     KillGLWindow(); 
     // Alles rckgngig machen
     MessageBox(NULL,"Konnte keinen DC erstellen.","ERROR",MB_OK|MB_ICONEXCLAMATION);
     return FALSE; // FALSE zurckgeben, beenden
   }


  if (!(PixelFormat=ChoosePixelFormat(hDC,&pfd))) 
  // Kann Windows ein passendes finden? 
  { 
	// Falls keins gefunden werden kann:
    KillGLWindow(); // Alles zurcksetzen
    MessageBox(NULL,"Konnte kein passendes Pixelformat finden.","ERROR",MB_OK|MB_ICONEXCLAMATION);
    return FALSE; // FALSE zurck und Ende.
  }


  if(!SetPixelFormat(hDC,PixelFormat,&pfd))
  // Kann das Pixelformat gesetzt werden?
    {
      KillGLWindow(); // Leider nicht, Fehlerpopup und raus
      MessageBox(NULL,"Konnte Pixelformat nicht setzen.","ERROR",MB_OK|MB_ICONEXCLAMATION);
      return FALSE; // FALSE zurck und raus
   }


  if (!(hRC=wglCreateContext(hDC))) // Versuch den RC zu bekommen
    {
      KillGLWindow(); // Alles rckgngig machen
      MessageBox(NULL,"Konnte keinen Rendering Context bekommen.","Fehler",MB_OK|MB_ICONEXCLAMATION);
      return FALSE;
    }



  if(!wglMakeCurrent(hDC,hRC)) // Versuch den RC zu aktivieren
  {
    KillGLWindow(); // hat nicht geklappt, also alles zurck
    MessageBox(NULL,"Konnte den Rendering Context nmicht aktivieren.","Fehler",MB_OK|MB_ICONEXCLAMATION);
    return FALSE;
  }



  ShowWindow(hWnd,SW_SHOW); // Fenster anzeigen
  SetForegroundWindow(hWnd); // Prioritt des Programms wird erhht
  SetFocus(hWnd); // Tastatureingaben werden 
                  // jetzt an das Programm geleitet

  ReSizeGLScene(width, height); // Die Perspektive wird aktiviert


  if (!InitGL()) // Initialisiere das OpenGL Fenster
  {
    KillGLWindow(); // Falls das nicht geklappt 
                    // haben sollte alles rckgngig machen

    MessageBox(NULL,"Initialisierung fehlgeschlagen.","Fehler",MB_OK|MB_ICONEXCLAMATION);
    return FALSE; // FALSE wird zurckgegeben
  }

  return TRUE; // Alles hat geklappt!!!
   
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{


  switch (uMsg) // Sind Nachrichten in der Nachrichtenschleife?
  {



    case WM_ACTIVATE: // Ist das Programm aktiv?
    {
      if (!HIWORD(wParam)) // Ist das Programm nicht minimiert?
      {
        active=TRUE; // active wird TRUE
      }

     else
     {
       active=FALSE; // Programm ist minimiert
     }   
  
     return 0; // Rckgabe: 0
   }



    case WM_SYSCOMMAND: // Ist ein Systemkommando 
                        // (wie z.B. "Bildschirmschoner 
                        // soll gestartet werden") vorhanden?
    {
      switch (wParam) // wParam wrde diesen Befehl enthalten
      {
        case SC_SCREENSAVE: 
        // Versucht Windows den Bildschirmschoner zu starten

        case SC_MONITORPOWER: 
        // Soll der Monitor in den Stromsparmodus gehen?
        
        return 0; 
        // Beide Flle werden durch die Rckgabe von 0 verhindert
      }
      break; // Das wars.
    }

    case WM_CLOSE: // Ist eine WM_CLOSE Nachricht vorhanden?
    {
      PostQuitMessage(0); 
      // Die Nachricht zum Beenden wird "gepostet"

      return 0; // und zurck.
    }

    case WM_KEYDOWN: // Drckt der Benutzer eine Taste???
    {
      keys[wParam] = TRUE; 
      // Der Wert im Array keys[] der dem Code 
      // der Taste entspricht, wird true gesetzt

      return 0; // und zurck...
    }

    case WM_KEYUP: // Wurde eine Taste losgelassen?
    {
      keys[wParam] = FALSE; 
      // Wen ja, dann soll dieser Wert im Array keys[] 
      // auf FALSE gesetzt werden
      return 0; // und zurck.
    }

    case WM_SIZE: // Die Fenstergre wurde gendert
    {
      ReSizeGLScene(LOWORD(lParam),HIWORD(lParam)); 
      // LoWord ist die Breite, HiWord die Hhe, 
      // ReSizeGLScene() verndert dann die 
      // Gre des OpenGL-Fensters
      return 0; // und zurck
    }
  }




  return DefWindowProc(hWnd,uMsg,wParam,lParam);
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{

  MSG msg;
  BOOL done=FALSE;


  if (MessageBox(NULL,"Soll im Vollbildmodus gestartet werden?","Vollbilmodus gewnscht?",MB_YESNO|MB_ICONQUESTION)==IDNO)
  {
    fullscreen=FALSE; // Falls nein gedrckt wurde, 
                      // wird fullscreen false gesetzt
  }


  if (!CreateGLWindow("Opengl Tutorial 5 - Teture Mapping - www.codeworx.org",640,480,16,fullscreen))
  {
    return 0; // Falls ein Fehler auftrat, beenden
  }


  while(!done) // Solange done nicht TRUE ist:
  {

    if (PeekMessage(&msg,NULL,0,0,PM_REMOVE)) 
    // Sind Nachrichten vorhanden
    {

    if (msg.message==WM_QUIT) 
    // Liegt eine Nachricht zum beenden vpr?
    {
      done=TRUE; // Wenn dem so ist, wird done 
                 // true und das Programm beendet
    }

    else 
    // Wenn nicht werden die anderen Nachrichten ausgewertet

    {
      TranslateMessage(&msg); // Umformen der Nachricht
      DispatchMessage(&msg); 
    }
  }

  else // Falls keine Nachrichten bereit liegen
  {

    if (active) // Programm aktiv?
    {
      if (keys[VK_ESCAPE]) // Wurde ESC gedrckt?
      {
        done=TRUE; // Wenn ESC gedrckt wurde, beenden
      }

      else // ESC wurde nicht gedrckt
      {

        DrawGLScene(); // Die Szene ausgeben
        SwapBuffers(hDC); // Die Puffer werden getauscht
      }
    }


    if (keys[VK_F1]) // Wird F1 gedrckt?
    {
      keys[VK_F1]=FALSE; 
      // Es darf nicht gewartet werden bis F1 losgelassen wird,
      // ansonsten knnte das Bild mehrmals hin und herschalten

      KillGLWindow(); // Das aktuelle Fenster wird gelscht
      fullscreen=!fullscreen; 
      // fullscreen erhlt seinen entgegengesetzten Wert
      // (bei FALSE TRUE, und bei TRUE FALSE.)

      if (!CreateGLWindow("openGL",640,480,16,fullscreen))    
      {
        return 0; // Zurck falls ein Fehler auftrat
      }
    }
  }
}



  KillGLWindow(); // Das Fenster lschen.
  return (msg.wParam); // Das Programm verlassen.
}
 

